;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; schematic-markdown.scm
;;;
;;; This script converts program source to markdown.
;;;
;;; It should run in any R7RS-compatible Scheme, so long as this
;;; project's lib directory is added to the include path. In Chibi,
;;; that's done with the -I flag.
;;;
;;; See this project's README for more information.
;;;
;;; Copyright (c) 2013, Evan Hanson <evhan@foldling.org>
;;; See LICENSE for details.
;;;

;; For this script.
(import (scheme base)
        (scheme read)
        (scheme file)
        (scheme process-context)
        (foldling command-line))

;; For schematic-reader.scm.
(import (scheme char)
        (scheme write)
        (scheme case-lambda))

(include "schematic-reader.scm")

;; The language is hardcoded to Lisp/Scheme for now.
(define comment-strings '(";;" ";;;"))

;; This should probably be replaced by the host's implementation.
(define (string-split str char)
  (let ((len (string-length str)))
    (let lp ((c 0)
             (a '()))
      (if (> c len)
          (reverse a)
          (do ((i c (+ i 1)))
              ((or (= i len)
                   (char=? (string-ref str i) char))
               (lp (+ i 1)
                   (cons (substring str c i) a))))))))

(define (process-input reader)
  (let-values (((docs code) (reader)))
    (unless (eof-object? docs)
      (display docs)
      (newline)
      (newline)
      (for-each
       (lambda (line)
         (display "    ")
         (display line)
         (newline))
       (string-split code #\newline))
      (newline)
      (process-input reader))))

(define (process-file file)
  (call-with-input-file file
   (lambda (port)
     (process-input
      (make-source-section-reader comment-strings port)))))

;; All command line arguments are treated as input files.
(let ((files (cdr (assq '-- (parse-command-line '())))))
  (if (pair? files)
      (for-each process-file files)
      (process-input (make-source-section-reader comment-strings))))
